@@ -33,6 +33,7 @@ from pandas.core.dtypes.common import (
     is_list_like)
 from pandas.io.formats.printing import pprint_thing
 from pandas.core.algorithms import take_1d
+from pandas.core.common import _all_not_none
 
 import pandas.compat as compat
 from pandas.compat import (
@@ -41,8 +42,6 @@ from pandas.compat import (
     StringIO, PY3
 )
 
-from pandas.core.computation import expressions as expr
-
 from pandas import (bdate_range, CategoricalIndex, Categorical, IntervalIndex,
                     DatetimeIndex, TimedeltaIndex, PeriodIndex, RangeIndex,
                     Index, MultiIndex,
@@ -50,20 +49,12 @@ from pandas import (bdate_range, CategoricalIndex, Categorical, IntervalIndex,
 
 from pandas._libs import testing as _testing
 from pandas.io.common import urlopen
-try:
-    import pytest
-    slow = pytest.mark.slow
-except ImportError:
-    # Should be ok to just ignore. If you actually need
-    # slow then you'll hit an import error long before getting here.
-    pass
 
 
 N = 30
 K = 4
 _RAISE_NETWORK_ERROR_DEFAULT = False
 
-
 # set testing_mode
 _testing_mode_warnings = (DeprecationWarning, compat.ResourceWarning)
 
@@ -111,7 +102,7 @@ def round_trip_pickle(obj, path=None):
     """
 
     if path is None:
-        path = u('__%s__.pickle' % rands(10))
+        path = u('__{random_bytes}__.pickle'.format(random_bytes=rands(10)))
     with ensure_clean(path) as path:
         pd.to_pickle(obj, path)
         return pd.read_pickle(path)
@@ -184,7 +175,7 @@ def assert_almost_equal(left, right, check_exact=False,
     ----------
     left : object
     right : object
-    check_exact : bool, default True
+    check_exact : bool, default False
         Whether to compare number exactly.
     check_dtype: bool, default True
         check dtype if both a and b are the same type
@@ -251,13 +242,15 @@ def _check_isinstance(left, right, cls):
     AssertionError : Either `left` or `right` is not an instance of `cls`.
     """
 
-    err_msg = "{0} Expected type {1}, found {2} instead"
+    err_msg = "{name} Expected type {exp_type}, found {act_type} instead"
     cls_name = cls.__name__
 
     if not isinstance(left, cls):
-        raise AssertionError(err_msg.format(cls_name, cls, type(left)))
+        raise AssertionError(err_msg.format(name=cls_name, exp_type=cls,
+                                            act_type=type(left)))
     if not isinstance(right, cls):
-        raise AssertionError(err_msg.format(cls_name, cls, type(right)))
+        raise AssertionError(err_msg.format(name=cls_name, exp_type=cls,
+                                            act_type=type(right)))
 
 
 def assert_dict_equal(left, right, compare_keys=True):
@@ -327,27 +320,18 @@ def close(fignum=None):
 
 
 def _skip_if_32bit():
-    import pytest
     if is_platform_32bit():
+        import pytest
         pytest.skip("skipping for 32 bit")
 
 
-def _skip_module_if_no_mpl():
+def _skip_if_no_mpl():
     import pytest
 
     mpl = pytest.importorskip("matplotlib")
     mpl.use("Agg", warn=False)
 
 
-def _skip_if_no_mpl():
-    try:
-        import matplotlib as mpl
-        mpl.use("Agg", warn=False)
-    except ImportError:
-        import pytest
-        pytest.skip("matplotlib not installed")
-
-
 def _skip_if_mpl_1_5():
     import matplotlib as mpl
 
@@ -360,21 +344,11 @@ def _skip_if_mpl_1_5():
 
 
 def _skip_if_no_scipy():
-    try:
-        import scipy.stats  # noqa
-    except ImportError:
-        import pytest
-        pytest.skip("no scipy.stats module")
-    try:
-        import scipy.interpolate  # noqa
-    except ImportError:
-        import pytest
-        pytest.skip('scipy.interpolate missing')
-    try:
-        import scipy.sparse  # noqa
-    except ImportError:
-        import pytest
-        pytest.skip('scipy.sparse missing')
+    import pytest
+
+    pytest.importorskip("scipy.stats")
+    pytest.importorskip("scipy.sparse")
+    pytest.importorskip("scipy.interpolate")
 
 
 def _check_if_lzma():
@@ -390,32 +364,14 @@ def _skip_if_no_lzma():
 
 
 def _skip_if_no_xarray():
-    try:
-        import xarray
-    except ImportError:
-        import pytest
-        pytest.skip("xarray not installed")
+    import pytest
 
+    xarray = pytest.importorskip("xarray")
     v = xarray.__version__
+
     if v < LooseVersion('0.7.0'):
         import pytest
-        pytest.skip("xarray not version is too low: {0}".format(v))
-
-
-def _skip_if_no_pytz():
-    try:
-        import pytz  # noqa
-    except ImportError:
-        import pytest
-        pytest.skip("pytz not installed")
-
-
-def _skip_if_no_dateutil():
-    try:
-        import dateutil  # noqa
-    except ImportError:
-        import pytest
-        pytest.skip("dateutil not installed")
+        pytest.skip("xarray version is too low: {version}".format(version=v))
 
 
 def _skip_if_windows_python_3():
@@ -446,21 +402,6 @@ def _skip_if_no_localpath():
         pytest.skip("py.path not installed")
 
 
-def _incompat_bottleneck_version(method):
-    """ skip if we have bottleneck installed
-    and its >= 1.0
-    as we don't match the nansum/nanprod behavior for all-nan
-    ops, see GH9422
-    """
-    if method not in ['sum', 'prod']:
-        return False
-    try:
-        import bottleneck as bn
-        return bn.__version__ >= LooseVersion('1.0')
-    except ImportError:
-        return False
-
-
 def skip_if_no_ne(engine='numexpr'):
     from pandas.core.computation.expressions import (
         _USE_NUMEXPR,
@@ -480,7 +421,7 @@ def _skip_if_has_locale():
     lang, _ = locale.getlocale()
     if lang is not None:
         import pytest
-        pytest.skip("Specific locale is set {0}".format(lang))
+        pytest.skip("Specific locale is set {lang}".format(lang=lang))
 
 
 def _skip_if_not_us_locale():
@@ -488,7 +429,7 @@ def _skip_if_not_us_locale():
     lang, _ = locale.getlocale()
     if lang != 'en_US':
         import pytest
-        pytest.skip("Specific locale is set {0}".format(lang))
+        pytest.skip("Specific locale is set {lang}".format(lang=lang))
 
 
 def _skip_if_no_mock():
@@ -498,16 +439,13 @@ def _skip_if_no_mock():
         try:
             from unittest import mock  # noqa
         except ImportError:
-            import nose
-            raise nose.SkipTest("mock is not installed")
+            import pytest
+            raise pytest.skip("mock is not installed")
 
 
 def _skip_if_no_ipython():
-    try:
-        import IPython  # noqa
-    except ImportError:
-        import nose
-        raise nose.SkipTest("IPython not installed")
+    import pytest
+    pytest.importorskip("IPython")
 
 # -----------------------------------------------------------------------------
 # locale utilities
@@ -552,8 +490,8 @@ def _default_locale_getter():
     try:
         raw_locales = check_output(['locale -a'], shell=True)
     except subprocess.CalledProcessError as e:
-        raise type(e)("%s, the 'locale -a' command cannot be found on your "
-                      "system" % e)
+        raise type(e)("{exception}, the 'locale -a' command cannot be found "
+                      "on your system".format(exception=e))
     return raw_locales
 
 
@@ -610,7 +548,8 @@ def get_locales(prefix=None, normalize=True,
     if prefix is None:
         return _valid_locales(out_locales, normalize)
 
-    found = re.compile('%s.*' % prefix).findall('\n'.join(out_locales))
+    found = re.compile('{prefix}.*'.format(prefix=prefix)) \
+              .findall('\n'.join(out_locales))
     return _valid_locales(found, normalize)
 
 
@@ -641,7 +580,7 @@ def set_locale(new_locale, lc_var=locale.LC_ALL):
         except ValueError:
             yield new_locale
         else:
-            if all(lc is not None for lc in normalized_locale):
+            if _all_not_none(*normalized_locale):
                 yield '.'.join(normalized_locale)
             else:
                 yield new_locale
@@ -865,13 +804,13 @@ def ensure_clean(filename=None, return_filelike=False):
             try:
                 os.close(fd)
             except Exception as e:
-                print("Couldn't close file descriptor: %d (file: %s)" %
-                      (fd, filename))
+                print("Couldn't close file descriptor: {fdesc} (file: {fname})"
+                      .format(fdesc=fd, fname=filename))
             try:
                 if os.path.exists(filename):
                     os.remove(filename)
             except Exception as e:
-                print("Exception on removing file: %s" % e)
+                print("Exception on removing file: {error}".format(error=e))
 
 
 def get_data_path(f=''):
@@ -947,15 +886,17 @@ def assert_index_equal(left, right, exact='equiv', check_names=True,
 
     # level comparison
     if left.nlevels != right.nlevels:
-        raise_assert_detail(obj, '{0} levels are different'.format(obj),
-                            '{0}, {1}'.format(left.nlevels, left),
-                            '{0}, {1}'.format(right.nlevels, right))
+        msg1 = '{obj} levels are different'.format(obj=obj)
+        msg2 = '{nlevels}, {left}'.format(nlevels=left.nlevels, left=left)
+        msg3 = '{nlevels}, {right}'.format(nlevels=right.nlevels, right=right)
+        raise_assert_detail(obj, msg1, msg2, msg3)
 
     # length comparison
     if len(left) != len(right):
-        raise_assert_detail(obj, '{0} length are different'.format(obj),
-                            '{0}, {1}'.format(len(left), left),
-                            '{0}, {1}'.format(len(right), right))
+        msg1 = '{obj} length are different'.format(obj=obj)
+        msg2 = '{length}, {left}'.format(length=len(left), left=left)
+        msg3 = '{length}, {right}'.format(length=len(right), right=right)
+        raise_assert_detail(obj, msg1, msg2, msg3)
 
     # MultiIndex special comparison for little-friendly error messages
     if left.nlevels > 1:
@@ -964,7 +905,7 @@ def assert_index_equal(left, right, exact='equiv', check_names=True,
             llevel = _get_ilevel_values(left, level)
             rlevel = _get_ilevel_values(right, level)
 
-            lobj = 'MultiIndex level [{0}]'.format(level)
+            lobj = 'MultiIndex level [{level}]'.format(level=level)
             assert_index_equal(llevel, rlevel,
                                exact=exact, check_names=check_names,
                                check_less_precise=check_less_precise,
@@ -976,8 +917,8 @@ def assert_index_equal(left, right, exact='equiv', check_names=True,
         if not left.equals(right):
             diff = np.sum((left.values != right.values)
                           .astype(int)) * 100.0 / len(left)
-            msg = '{0} values are different ({1} %)'\
-                .format(obj, np.round(diff, 5))
+            msg = '{obj} values are different ({pct} %)'.format(
+                obj=obj, pct=np.round(diff, 5))
             raise_assert_detail(obj, msg, left, right)
     else:
         _testing.assert_almost_equal(left.values, right.values,
@@ -997,7 +938,7 @@ def assert_index_equal(left, right, exact='equiv', check_names=True,
     if check_categorical:
         if is_categorical_dtype(left) or is_categorical_dtype(right):
             assert_categorical_equal(left.values, right.values,
-                                     obj='{0} category'.format(obj))
+                                     obj='{obj} category'.format(obj=obj))
 
 
 def assert_class_equal(left, right, exact=True, obj='Input'):
@@ -1018,12 +959,12 @@ def assert_class_equal(left, right, exact=True, obj='Input'):
             # allow equivalence of Int64Index/RangeIndex
             types = set([type(left).__name__, type(right).__name__])
             if len(types - set(['Int64Index', 'RangeIndex'])):
-                msg = '{0} classes are not equivalent'.format(obj)
+                msg = '{obj} classes are not equivalent'.format(obj=obj)
                 raise_assert_detail(obj, msg, repr_class(left),
                                     repr_class(right))
     elif exact:
         if type(left) != type(right):
-            msg = '{0} classes are different'.format(obj)
+            msg = '{obj} classes are different'.format(obj=obj)
             raise_assert_detail(obj, msg, repr_class(left),
                                 repr_class(right))
 
@@ -1063,23 +1004,22 @@ def assert_attr_equal(attr, left, right, obj='Attributes'):
     if result:
         return True
     else:
-        raise_assert_detail(obj, 'Attribute "{0}" are different'.format(attr),
-                            left_attr, right_attr)
+        msg = 'Attribute "{attr}" are different'.format(attr=attr)
+        raise_assert_detail(obj, msg, left_attr, right_attr)
 
 
 def assert_is_valid_plot_return_object(objs):
     import matplotlib.pyplot as plt
     if isinstance(objs, (pd.Series, np.ndarray)):
         for el in objs.ravel():
-            msg = ('one of \'objs\' is not a matplotlib Axes instance, '
-                   'type encountered {0!r}')
-            assert isinstance(el, (plt.Axes, dict)), msg.format(
-                el.__class__.__name__)
+            msg = ('one of \'objs\' is not a matplotlib Axes instance, type '
+                   'encountered {name!r}').format(name=el.__class__.__name__)
+            assert isinstance(el, (plt.Axes, dict)), msg
     else:
         assert isinstance(objs, (plt.Artist, tuple, dict)), \
             ('objs is neither an ndarray of Artist instances nor a '
-             'single Artist instance, tuple, or dict, "objs" is a {0!r} '
-             ''.format(objs.__class__.__name__))
+             'single Artist instance, tuple, or dict, "objs" is a {name!r}'
+             ).format(name=objs.__class__.__name__)
 
 
 def isiterable(obj):
@@ -1116,17 +1056,17 @@ def assert_categorical_equal(left, right, check_dtype=True,
 
     if check_category_order:
         assert_index_equal(left.categories, right.categories,
-                           obj='{0}.categories'.format(obj))
+                           obj='{obj}.categories'.format(obj=obj))
         assert_numpy_array_equal(left.codes, right.codes,
                                  check_dtype=check_dtype,
-                                 obj='{0}.codes'.format(obj))
+                                 obj='{obj}.codes'.format(obj=obj))
     else:
         assert_index_equal(left.categories.sort_values(),
                            right.categories.sort_values(),
-                           obj='{0}.categories'.format(obj))
+                           obj='{obj}.categories'.format(obj=obj))
         assert_index_equal(left.categories.take(left.codes),
                            right.categories.take(right.codes),
-                           obj='{0}.values'.format(obj))
+                           obj='{obj}.values'.format(obj=obj))
 
     assert_attr_equal('ordered', left, right, obj=obj)
 
@@ -1137,14 +1077,14 @@ def raise_assert_detail(obj, message, left, right, diff=None):
     if isinstance(right, np.ndarray):
         right = pprint_thing(right)
 
-    msg = """{0} are different
+    msg = """{obj} are different
 
-{1}
-[left]:  {2}
-[right]: {3}""".format(obj, message, left, right)
+{message}
+[left]:  {left}
+[right]: {right}""".format(obj=obj, message=message, left=left, right=right)
 
     if diff is not None:
-        msg = msg + "\n[diff]: {diff}".format(diff=diff)
+        msg += "\n[diff]: {diff}".format(diff=diff)
 
     raise AssertionError(msg)
 
@@ -1185,18 +1125,20 @@ def assert_numpy_array_equal(left, right, strict_nan=False,
 
     if check_same == 'same':
         if left_base is not right_base:
-            msg = "%r is not %r" % (left_base, right_base)
+            msg = "{left!r} is not {right!r}".format(
+                left=left_base, right=right_base)
             raise AssertionError(msg)
     elif check_same == 'copy':
         if left_base is right_base:
-            msg = "%r is %r" % (left_base, right_base)
+            msg = "{left!r} is {right!r}".format(
+                left=left_base, right=right_base)
             raise AssertionError(msg)
 
     def _raise(left, right, err_msg):
         if err_msg is None:
             if left.shape != right.shape:
-                raise_assert_detail(obj, '{0} shapes are different'
-                                    .format(obj), left.shape, right.shape)
+                raise_assert_detail(obj, '{obj} shapes are different'
+                                    .format(obj=obj), left.shape, right.shape)
 
             diff = 0
             for l, r in zip(left, right):
@@ -1205,8 +1147,8 @@ def assert_numpy_array_equal(left, right, strict_nan=False,
                     diff += 1
 
             diff = diff * 100.0 / left.size
-            msg = '{0} values are different ({1} %)'\
-                .format(obj, np.round(diff, 5))
+            msg = '{obj} values are different ({pct} %)'.format(
+                obj=obj, pct=np.round(diff, 5))
             raise_assert_detail(obj, msg, left, right)
 
         raise AssertionError(err_msg)
@@ -1273,9 +1215,9 @@ def assert_series_equal(left, right, check_dtype=True,
 
     # length comparison
     if len(left) != len(right):
-        raise_assert_detail(obj, 'Series length are different',
-                            '{0}, {1}'.format(len(left), left.index),
-                            '{0}, {1}'.format(len(right), right.index))
+        msg1 = '{len}, {left}'.format(len=len(left), left=left.index)
+        msg2 = '{len}, {right}'.format(len=len(right), right=right.index)
+        raise_assert_detail(obj, 'Series length are different', msg1, msg2)
 
     # index comparison
     assert_index_equal(left.index, right.index, exact=check_index_type,
@@ -1283,15 +1225,22 @@ def assert_series_equal(left, right, check_dtype=True,
                        check_less_precise=check_less_precise,
                        check_exact=check_exact,
                        check_categorical=check_categorical,
-                       obj='{0}.index'.format(obj))
+                       obj='{obj}.index'.format(obj=obj))
 
     if check_dtype:
-        assert_attr_equal('dtype', left, right)
+        # We want to skip exact dtype checking when `check_categorical`
+        # is False. We'll still raise if only one is a `Categorical`,
+        # regardless of `check_categorical`
+        if (is_categorical_dtype(left) and is_categorical_dtype(right) and
+                not check_categorical):
+            pass
+        else:
+            assert_attr_equal('dtype', left, right)
 
     if check_exact:
         assert_numpy_array_equal(left.get_values(), right.get_values(),
                                  check_dtype=check_dtype,
-                                 obj='{0}'.format(obj),)
+                                 obj='{obj}'.format(obj=obj),)
     elif check_datetimelike_compat:
         # we want to check only if we have compat dtypes
         # e.g. integer and M|m are NOT compat, but we can simply check
@@ -1304,8 +1253,9 @@ def assert_series_equal(left, right, check_dtype=True,
             # datetimelike may have different objects (e.g. datetime.datetime
             # vs Timestamp) but will compare equal
             if not Index(left.values).equals(Index(right.values)):
-                msg = '[datetimelike_compat=True] {0} is not equal to {1}.'
-                raise AssertionError(msg.format(left.values, right.values))
+                msg = ('[datetimelike_compat=True] {left} is not equal to '
+                       '{right}.').format(left=left.values, right=right.values)
+                raise AssertionError(msg)
         else:
             assert_numpy_array_equal(left.get_values(), right.get_values(),
                                      check_dtype=check_dtype)
@@ -1313,13 +1263,13 @@ def assert_series_equal(left, right, check_dtype=True,
         # TODO: big hack here
         l = pd.IntervalIndex(left)
         r = pd.IntervalIndex(right)
-        assert_index_equal(l, r, obj='{0}.index'.format(obj))
+        assert_index_equal(l, r, obj='{obj}.index'.format(obj=obj))
 
     else:
         _testing.assert_almost_equal(left.get_values(), right.get_values(),
                                      check_less_precise=check_less_precise,
                                      check_dtype=check_dtype,
-                                     obj='{0}'.format(obj))
+                                     obj='{obj}'.format(obj=obj))
 
     # metadata comparison
     if check_names:
@@ -1328,7 +1278,7 @@ def assert_series_equal(left, right, check_dtype=True,
     if check_categorical:
         if is_categorical_dtype(left) or is_categorical_dtype(right):
             assert_categorical_equal(left.values, right.values,
-                                     obj='{0} category'.format(obj))
+                                     obj='{obj} category'.format(obj=obj))
 
 
 # This could be refactored to use the NDFrame.equals method
@@ -1395,8 +1345,8 @@ def assert_frame_equal(left, right, check_dtype=True,
     if left.shape != right.shape:
         raise_assert_detail(obj,
                             'DataFrame shape mismatch',
-                            '({0}, {1})'.format(*left.shape),
-                            '({0}, {1})'.format(*right.shape))
+                            '{shape!r}'.format(shape=left.shape),
+                            '{shape!r}'.format(shape=right.shape))
 
     if check_like:
         left, right = left.reindex_like(right), right
@@ -1407,7 +1357,7 @@ def assert_frame_equal(left, right, check_dtype=True,
                        check_less_precise=check_less_precise,
                        check_exact=check_exact,
                        check_categorical=check_categorical,
-                       obj='{0}.index'.format(obj))
+                       obj='{obj}.index'.format(obj=obj))
 
     # column comparison
     assert_index_equal(left.columns, right.columns, exact=check_column_type,
@@ -1415,12 +1365,12 @@ def assert_frame_equal(left, right, check_dtype=True,
                        check_less_precise=check_less_precise,
                        check_exact=check_exact,
                        check_categorical=check_categorical,
-                       obj='{0}.columns'.format(obj))
+                       obj='{obj}.columns'.format(obj=obj))
 
     # compare by blocks
     if by_blocks:
-        rblocks = right.blocks
-        lblocks = left.blocks
+        rblocks = right._to_dict_of_blocks()
+        lblocks = left._to_dict_of_blocks()
         for dtype in list(set(list(lblocks.keys()) + list(rblocks.keys()))):
             assert dtype in lblocks
             assert dtype in rblocks
@@ -1440,7 +1390,7 @@ def assert_frame_equal(left, right, check_dtype=True,
                 check_exact=check_exact, check_names=check_names,
                 check_datetimelike_compat=check_datetimelike_compat,
                 check_categorical=check_categorical,
-                obj='DataFrame.iloc[:, {0}]'.format(i))
+                obj='DataFrame.iloc[:, {idx}]'.format(idx=i))
 
 
 def assert_panelnd_equal(left, right,
@@ -1495,13 +1445,15 @@ def assert_panelnd_equal(left, right,
 
         # can potentially be slow
         for i, item in enumerate(left._get_axis(0)):
-            assert item in right, "non-matching item (right) '%s'" % item
+            msg = "non-matching item (right) '{item}'".format(item=item)
+            assert item in right, msg
             litem = left.iloc[i]
             ritem = right.iloc[i]
             assert_func(litem, ritem, check_less_precise=check_less_precise)
 
         for i, item in enumerate(right._get_axis(0)):
-            assert item in left, "non-matching item (left) '%s'" % item
+            msg = "non-matching item (left) '{item}'".format(item=item)
+            assert item in left, msg
 
 
 # TODO: strangely check_names fails in py3 ?
@@ -1573,7 +1525,7 @@ def assert_sp_series_equal(left, right, check_dtype=True, exact_indices=True,
         assert_class_equal(left, right, obj=obj)
 
     assert_index_equal(left.index, right.index,
-                       obj='{0}.index'.format(obj))
+                       obj='{obj}.index'.format(obj=obj))
 
     assert_sp_array_equal(left.block.values, right.block.values)
 
@@ -1610,9 +1562,9 @@ def assert_sp_frame_equal(left, right, check_dtype=True, exact_indices=True,
         assert_class_equal(left, right, obj=obj)
 
     assert_index_equal(left.index, right.index,
-                       obj='{0}.index'.format(obj))
+                       obj='{obj}.index'.format(obj=obj))
     assert_index_equal(left.columns, right.columns,
-                       obj='{0}.columns'.format(obj))
+                       obj='{obj}.columns'.format(obj=obj))
 
     for col, series in compat.iteritems(left):
         assert (col in right)
@@ -1646,7 +1598,7 @@ def assert_sp_list_equal(left, right):
 
 def assert_contains_all(iterable, dic):
     for k in iterable:
-        assert k in dic, "Did not contain item: '%r'" % k
+        assert k in dic, "Did not contain item: '{key!r}'".format(key=k)
 
 
 def assert_copy(iter1, iter2, **eql_kwargs):
@@ -1660,10 +1612,10 @@ def assert_copy(iter1, iter2, **eql_kwargs):
     """
     for elem1, elem2 in zip(iter1, iter2):
         assert_almost_equal(elem1, elem2, **eql_kwargs)
-        assert elem1 is not elem2, ("Expected object %r and "
-                                    "object %r to be different "
-                                    "objects, were same."
-                                    % (type(elem1), type(elem2)))
+        msg = ("Expected object {obj1!r} and object {obj2!r} to be "
+               "different objects, but they were the same object."
+               ).format(obj1=type(elem1), obj2=type(elem2))
+        assert elem1 is not elem2, msg
 
 
 def getCols(k):
@@ -1842,22 +1794,24 @@ def makePeriodFrame(nper=None):
 
 
 def makePanel(nper=None):
-    cols = ['Item' + c for c in string.ascii_uppercase[:K - 1]]
-    data = dict((c, makeTimeDataFrame(nper)) for c in cols)
-    return Panel.fromDict(data)
+    with warnings.catch_warnings(record=True):
+        cols = ['Item' + c for c in string.ascii_uppercase[:K - 1]]
+        data = dict((c, makeTimeDataFrame(nper)) for c in cols)
+        return Panel.fromDict(data)
 
 
 def makePeriodPanel(nper=None):
-    cols = ['Item' + c for c in string.ascii_uppercase[:K - 1]]
-    data = dict((c, makePeriodFrame(nper)) for c in cols)
-    return Panel.fromDict(data)
+    with warnings.catch_warnings(record=True):
+        cols = ['Item' + c for c in string.ascii_uppercase[:K - 1]]
+        data = dict((c, makePeriodFrame(nper)) for c in cols)
+        return Panel.fromDict(data)
 
 
 def makePanel4D(nper=None):
     with warnings.catch_warnings(record=True):
         d = dict(l1=makePanel(nper), l2=makePanel(nper),
                  l3=makePanel(nper))
-    return Panel4D(d)
+        return Panel4D(d)
 
 
 def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
@@ -1915,8 +1869,9 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
             idx.name = names[0]
         return idx
     elif idx_type is not None:
-        raise ValueError('"%s" is not a legal value for `idx_type`, use  '
-                         '"i"/"f"/"s"/"u"/"dt/"p"/"td".' % idx_type)
+        raise ValueError('"{idx_type}" is not a legal value for `idx_type`, '
+                         'use  "i"/"f"/"s"/"u"/"dt/"p"/"td".'
+                         .format(idx_type=idx_type))
 
     if len(ndupe_l) < nlevels:
         ndupe_l.extend([1] * (nlevels - len(ndupe_l)))
@@ -1928,14 +1883,14 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
     for i in range(nlevels):
         def keyfunc(x):
             import re
-            numeric_tuple = re.sub("[^\d_]_?", "", x).split("_")
+            numeric_tuple = re.sub(r"[^\d_]_?", "", x).split("_")
             return lmap(int, numeric_tuple)
 
         # build a list of lists to create the index from
         div_factor = nentries // ndupe_l[i] + 1
         cnt = Counter()
         for j in range(div_factor):
-            label = prefix + '_l%d_g' % i + str(j)
+            label = '{prefix}_l{i}_g{j}'.format(prefix=prefix, i=i, j=j)
             cnt[label] = ndupe_l[i]
         # cute Counter trick
         result = list(sorted(cnt.elements(), key=keyfunc))[:nentries]
@@ -1945,7 +1900,11 @@ def makeCustomIndex(nentries, nlevels, prefix='#', names=False, ndupe_l=None,
 
     # convert tuples to index
     if nentries == 1:
+        # we have a single level of tuples, i.e. a regular Index
         index = Index(tuples[0], name=names[0])
+    elif nlevels == 1:
+        name = None if names is None else names[0]
+        index = Index((x[0] for x in tuples), name=name)
     else:
         index = MultiIndex.from_tuples(tuples, names=names)
     return index
@@ -2028,7 +1987,7 @@ def makeCustomDataframe(nrows, ncols, c_idx_names=True, r_idx_names=True,
 
     # by default, generate data based on location
     if data_gen_f is None:
-        data_gen_f = lambda r, c: "R%dC%d" % (r, c)
+        data_gen_f = lambda r, c: "R{rows}C{cols}".format(rows=r, cols=c)
 
     data = [[data_gen_f(r, c) for c in range(ncols)] for r in range(nrows)]
 
@@ -2157,13 +2116,13 @@ def skip_if_no_package(pkg_name, min_version=None, max_version=None,
 
     import pytest
     if app:
-        msg = '%s requires %s' % (app, pkg_name)
+        msg = '{app} requires {pkg_name}'.format(app=app, pkg_name=pkg_name)
     else:
-        msg = 'module requires %s' % pkg_name
+        msg = 'module requires {pkg_name}'.format(pkg_name=pkg_name)
     if min_version:
-        msg += ' with version >= %s' % (min_version,)
+        msg += ' with version >= {min_version}'.format(min_version=min_version)
     if max_version:
-        msg += ' with version < %s' % (max_version,)
+        msg += ' with version < {max_version}'.format(max_version=max_version)
     try:
         mod = __import__(pkg_name)
     except ImportError:
@@ -2171,7 +2130,8 @@ def skip_if_no_package(pkg_name, min_version=None, max_version=None,
     try:
         have_version = mod.__version__
     except AttributeError:
-        pytest.skip('Cannot find version for %s' % pkg_name)
+        pytest.skip('Cannot find version for {pkg_name}'
+                    .format(pkg_name=pkg_name))
     if min_version and checker(have_version) < checker(min_version):
         pytest.skip(msg)
     if max_version and checker(have_version) >= checker(max_version):
@@ -2377,7 +2337,7 @@ def network(t, url="http://www.google.com",
 
             if errno in skip_errnos:
                 skip("Skipping test due to known errno"
-                     " and error %s" % e)
+                     " and error {error}".format(error=e))
 
             try:
                 e_str = traceback.format_exc(e)
@@ -2386,7 +2346,7 @@ def network(t, url="http://www.google.com",
 
             if any([m.lower() in e_str.lower() for m in _skip_on_messages]):
                 skip("Skipping test because exception "
-                     "message is known and error %s" % e)
+                     "message is known and error {error}".format(error=e))
 
             if not isinstance(e, error_classes):
                 raise
@@ -2395,7 +2355,7 @@ def network(t, url="http://www.google.com",
                 raise
             else:
                 skip("Skipping test due to lack of connectivity"
-                     " and error %s" % e)
+                     " and error {error}".format(e))
 
     return wrapper
 
@@ -2458,19 +2418,12 @@ def stdin_encoding(encoding=None):
 
 def assert_raises_regex(_exception, _regexp, _callable=None,
                         *args, **kwargs):
-    """
+    r"""
     Check that the specified Exception is raised and that the error message
     matches a given regular expression pattern. This may be a regular
     expression object or a string containing a regular expression suitable
-    for use by `re.search()`.
-
-    This is a port of the `assertRaisesRegexp` function from unittest in
-    Python 2.7. However, with our migration to `pytest`, please refrain
-    from using this. Instead, use the following paradigm:
-
-    with pytest.raises(_exception) as exc_info:
-       func(*args, **kwargs)
-    exc_info.matches(reg_exp)
+    for use by `re.search()`. This is a port of the `assertRaisesRegexp`
+    function from unittest in Python 2.7.
 
     Examples
     --------
@@ -2540,7 +2493,7 @@ class _AssertRaisesContextmanager(object):
 
         if not exc_type:
             exp_name = getattr(expected, "__name__", str(expected))
-            raise AssertionError("{0} not raised.".format(exp_name))
+            raise AssertionError("{name} not raised.".format(name=exp_name))
 
         return self.exception_matches(exc_type, exc_value, trace_back)
 
@@ -2575,8 +2528,9 @@ class _AssertRaisesContextmanager(object):
                 val = str(exc_value)
 
                 if not self.regexp.search(val):
-                    e = AssertionError('"%s" does not match "%s"' %
-                                       (self.regexp.pattern, str(val)))
+                    msg = '"{pat}" does not match "{val}"'.format(
+                        pat=self.regexp.pattern, val=val)
+                    e = AssertionError(msg)
                     raise_with_traceback(e, trace_back)
 
             return True
@@ -2643,18 +2597,20 @@ def assert_produces_warning(expected_warning=Warning, filter_level="always",
                     from inspect import getframeinfo, stack
                     caller = getframeinfo(stack()[2][0])
                     msg = ("Warning not set with correct stacklevel. "
-                           "File where warning is raised: {0} != {1}. "
-                           "Warning message: {2}".format(
-                               actual_warning.filename, caller.filename,
-                               actual_warning.message))
+                           "File where warning is raised: {actual} != "
+                           "{caller}. Warning message: {message}"
+                           ).format(actual=actual_warning.filename,
+                                    caller=caller.filename,
+                                    message=actual_warning.message)
                     assert actual_warning.filename == caller.filename, msg
             else:
                 extra_warnings.append(actual_warning.category.__name__)
         if expected_warning:
-            assert saw_warning, ("Did not see expected warning of class %r."
-                                 % expected_warning.__name__)
-        assert not extra_warnings, ("Caused unexpected warning(s): %r."
-                                    % extra_warnings)
+            msg = "Did not see expected warning of class {name!r}.".format(
+                name=expected_warning.__name__)
+            assert saw_warning, msg
+        assert not extra_warnings, ("Caused unexpected warning(s): {extra!r}."
+                                    ).format(extra=extra_warnings)
 
 
 class RNGContext(object):
@@ -2688,7 +2644,11 @@ class RNGContext(object):
 
 
 @contextmanager
-def use_numexpr(use, min_elements=expr._MIN_ELEMENTS):
+def use_numexpr(use, min_elements=None):
+    from pandas.core.computation import expressions as expr
+    if min_elements is None:
+        min_elements = expr._MIN_ELEMENTS
+
     olduse = expr._USE_NUMEXPR
     oldmin = expr._MIN_ELEMENTS
     expr.set_use_numexpr(use)
